/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElementNS;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTask;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTest;

import java.io.File;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


/**
 * This class represents an Ant task  that extends the optional JUnit task to provide support for testing using this
 * Component. This is the main Ant task  of this component.
 * <p/>
 * <ul><li>Supported attributes: <ul><li><tt>reportsdir</tt> - not required, specifies the directory to which the
 * reports of the test run should be written</li></ul> </li><li>Nested  XML elements: <ul><li>One or more dynamic (i.e.
 * with free element name instead of defined name) elements where the element name specifies the fully qualified class
 * name to be used for instantiating the child element of type {@link Server}</li></ul></li></ul>
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class ServerAppTestTask extends JUnitTask implements DynamicElementNS {

    /**
     * This field represents the list of server elements. All elements of the list should be  ServerElement instances.
     * Is modified by {@link #createServer()} method. Then the list will be used in {@link #execute()} method.
     */
    private final List servers = new LinkedList();

    /**
     * This field represents the <tt>reportsdir</tt> attribute, which represents the directory to which the test reports
     * should be written. The field is initialized to <tt>null</tt> and set via the {@link #setReportsDir(java.io.File)}
     * method.
     */
    private File reportsDir = null;

    /**
     * Creates a ServerAppTestTask instance.
     *
     * @throws Exception in case the construction fails
     */
    public ServerAppTestTask() throws Exception {
        super();
    }

    /**
     * Executes the task. First starts the servers if needed. Then runs the testcases. After running  the testcases the
     * servers are stopped.
     *
     * @throws BuildException if any error happens
     */
    public void execute() {
        if (servers.isEmpty()) {
            throw new BuildException("No server child elements have been specified.");
        }
        try {
            //initialize all servers
            for (Iterator iterator = servers.iterator(); iterator.hasNext();) {
                final Server server = ((ServerElement) iterator.next()).getServer();
                server.startUp();
                server.initData();
            }

            //run the tests
            final Enumeration individualTests = getIndividualTests();
            while (individualTests.hasMoreElements()) {
                final JUnitTest jUnitTest = (JUnitTest) individualTests.nextElement();
                if (jUnitTest.shouldRun(getProject())) {
                    //set output dir
                    if (getReportsDir() != null) {
                        jUnitTest.setTodir(getReportsDir());
                    }
                    //execute test
                    execute(jUnitTest);
                }
            }

            for (Iterator iterator = servers.iterator(); iterator.hasNext();) {
                final Server server = ((ServerElement) iterator.next()).getServer();
                server.cleanUpData();
                server.shutDown();
            }
        } catch (Exception e) {
            // Catch of Exception is normally discouraged by TC style, but as we want
            // to 'shield' the caller from the internals of the wrapped implementation
            // of cactus (which sometimes can throw NPEs) we simply wrap everything
            // into BuildException to assure normal ant behavior
            //
            // This method does not do any 'clever' finally-block cleanup strategies
            // as this would only complicate the debugging of errors in tests or setup.
            // see https://software.topcoder.com/forum/c_forum_message.jsp?f=20015788&r=21585282
            throw new BuildException("Error while executing the tests.", e);
        }
    }

    /**
     * This method creates an element with the given name. Actually it creates a {@link ServerElement} for each call to
     * this method and given the argument of this method to the ServerElement as constructor arg. The ServerElement
     * expects this element to be a fully qualified class name of a {@link Server} subclass.
     *
     * @param uri       The namespace uri for this element
     * @param localName The localname of this element
     * @param qName     The qualified name for this element
     *
     * @return the element created for this element
     *
     * @throws org.apache.tools.ant.BuildException
     *                                  when any error occurs
     * @throws IllegalArgumentException in case localName is <tt>null</tt> or an empty (trim'd) String
     */
    public Object createDynamicElement(final String uri, final String localName, final String qName) {
        if (localName == null) {
            throw new IllegalArgumentException("The parameter named [localName] was null.");
        }
        if (localName.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [localName] was an empty String.");
        }

        final ServerElement serverElement = new ServerElement(localName);
        serverElement.setAntTaskInfo(new AntTaskInfo(getProject(), getTaskName(), getLocation(), getOwningTarget()));
        servers.add(serverElement);
        return serverElement;
    }

    /**
     * This method sets the value of the <tt>reportsdir</tt> attribute, which represents the directory to which the test
     * reports should be written.
     *
     * @param reportsDir the value of the <tt>reportsdir</tt> attribute
     */
    public void setReportsDir(final File reportsDir) {
        this.reportsDir = reportsDir;
    }

    /**
     * This method returns the value of the <tt>reportsdir</tt> attribute, which represents the directory to which the
     * test reports should be written.
     *
     * @return the value of the <tt>reportsdir</tt> attribute
     */
    public File getReportsDir() {
        return reportsDir;
    }

}
