/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import com.topcoder.testframework.AntTaskInfo;
import com.topcoder.testframework.ApplicationServer;
import org.apache.cactus.integration.ant.container.Container;
import org.apache.cactus.integration.ant.container.ContainerFactory;
import org.apache.cactus.integration.ant.container.ContainerRunner;
import org.apache.cactus.integration.ant.deployment.DeployableFile;
import org.apache.cactus.integration.ant.deployment.EarParser;
import org.apache.cactus.integration.ant.deployment.WarParser;
import org.apache.cactus.integration.ant.util.DefaultAntTaskFactory;
import org.apache.tools.ant.BuildException;

import java.io.File;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;


/**
 * This class is derived from {@link ApplicationServer} class. It represents web application server to be used while
 * running tests. It uses Cactus as a back-end. It tries to support all the servers supported by Cactus. Name of
 * corresponding XML element in the Ant build file should be equal to the Cactus container type representing the desired
 * server.
 * <p/>
 * <b>Supported attributes:</b> <ul><li><tt>timeout</tt> - non-required, specifies the timeout to wait for successful
 * server startup in milliseconds, default value is 180000 milliseconds</li> <li><tt>testurl</tt> - non-required,
 * specifies a url to be used to test whether the server has started up and is available, if not specified will be
 * calculated from the prepared deployable (war or ear) file and server startup parameters</li></ul>
 * <p/>
 * This class is mutable, i.e. not thread-safe.
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class DefaultWebApplicationServer extends ApplicationServer {
    /**
     * This is the default timeout used when no timeout has been specified. It is 180000 ms, i.e. 3 minutes.
     */
    private static final int DEFAULT_TIMEOUT = 180000;

    /**
     * This field represents the Cactus container representing the server to be used when running the tests. Is
     * initialized in constructor, using {@link org.apache.cactus.integration.ant.container.ContainerFactory} to create
     * an instance of the class.
     */
    private Container container;

    /**
     * This field stores instance of Cactus ContainerRunner class to be used for starting/stopping the server
     * represented by the container field.
     */
    private ContainerRunner runner;

    /**
     * This field represents the <tt>timeout</tt> attribute, which specifies the amount of time to wait for server to be
     * started (in milliseconds). The field is initialized to <tt>180000</tt> and set via the {@link #setTimeout(long)}
     * method.
     */
    private long timeout = DEFAULT_TIMEOUT;

    /**
     * This field represents the <tt>testurl</tt> attribute, which specifies the URL used to check whether the server is
     * running.
     * <p/>
     * This attribute is required only when neither <tt>warfile</tt>  nor <tt>earfile</tt> attributes are specified. If
     * this attribute is set and the server is detected to be running, the server will not be started by the framework.
     * The field is initialized to <tt>null</tt> and set via the {@link #setTestURL(java.net.URL)} method.
     */
    private URL testURL = null;

    /**
     * Creates a DefaultWebApplicationServer instance. The type of server represented by the class is specified. Type is
     * a string like <tt>"tomcat5x"</tt>, which actually specifies the name and version of the server.
     *
     * @param type the type of the server
     *
     * @throws BuildException           if the type of the server is unrecognized, or any other error happens
     * @throws IllegalArgumentException if type is <tt>null</tt>
     */
    public DefaultWebApplicationServer(final String type) {
        //arg checking done in super method
        super(type);
        final ContainerFactory containerFactory = new ContainerFactory();
        container = containerFactory.createContainer(type);
        runner = new ContainerRunner(container);
    }

    /**
     * This method starts the server. In the case when <tt>startserver</tt> attribute is set to <tt>false</tt>, the
     * server should not be started, but some initialization may still be needed.
     * <p/>
     * In case there is a valid <tt>testurl</tt> attribute set on this instance, this will be used to test if the server
     * is already running. If it is running , then it is not started and stopped by the framework.
     *
     * @throws BuildException if any error happens
     */
    public void startUp() {
        final File configuredWarFile = getWarFile();
        final File configuredEarFile = getEarFile();

        if ((configuredWarFile != null) && (configuredEarFile != null)) {
            throw new BuildException("You must specify either the [warfile] or "
                + "the [earfile] attribute but not both");
        }

        // Get DeployableFile instance
        DeployableFile deployableFile = null;
        if (configuredWarFile != null) {
            deployableFile = WarParser.parse(configuredWarFile);
        } else if (configuredEarFile != null) {
            deployableFile = EarParser.parse(configuredEarFile);
        }

        // Retrieve Ant task information
        final AntTaskInfo taskInfo = getAntTaskInfo();
        // Create and set ant task factory
        container.setAntTaskFactory(new DefaultAntTaskFactory(taskInfo.getProject(), taskInfo.getTaskName(),
            taskInfo.getLocation(), taskInfo.getTarget()));

        // Set deployable file
        container.setDeployableFile(deployableFile);

        callSetterByReflection(container, "setServer", String.class, getServer());
        if (getPort() > 0) {
            callSetterByReflection(container, "setPort", int.class, new Integer(getPort()));
        }
        callSetterByReflection(container, "setDir", File.class, getDir());

        // Init the container
        container.init();
        // Set URL to test connection with
        if (getTestURL() != null) {
            runner.setURL(getTestURL());
            // according to https://software.topcoder.com/forum/c_forum_message.jsp?f=20015788&r=21551343
            // we set the context url to be the testURL
            System.setProperty("cactus.contextURL", getTestURL().toExternalForm());
        } else if (deployableFile != null) {
            final String spec = container.getBaseURL() + "/"
                + deployableFile.getTestContext()
                + deployableFile.getServletRedirectorMapping()
                + "?Cactus_Service=RUN_TEST";
            try {
                runner.setURL(new URL(spec));
            } catch (MalformedURLException e) {
                throw new BuildException("The URL spec [" + spec + "] could not be converted into a valid URL.", e);
            }
            // sets the URL to be used by the test case client when
            // connecting to the container to call the container-side test cases
            // see https://software.topcoder.com/forum/c_forum_message.jsp?f=20015788&r=21553769
            System.setProperty("cactus.contextURL", container.getBaseURL() + "/" + deployableFile.getTestContext());
        } else {
            throw new BuildException("Unable to start the container: Neither a valid 'testUrl' attribute nor "
                + "a deployable file ('warFile' or 'earFile') were specified.");
        }

        // Set timeout
        runner.setTimeout(getTimeout());

        // Start the container (starts server if server wasn't already running)
        runner.startUpContainer();
    }

    /**
     * This method stops the server, and possibly clean-ups the data. The server should not be stopped if it wasn't
     * started by the {@link #startUp()} method.
     *
     * @throws BuildException if any error happens
     */
    public void shutDown() {
        runner.shutDownContainer();
    }

    /**
     * This method initializes the server with data. In the case when <tt>initserver</tt> attribute is set to
     * <tt>false</tt>, does nothing, or can check if the data are properly initialized, the implementation can choose
     * the preferred way. Subclasses must implement this method to perform the actual task of initializing server with
     * data.
     *
     * @throws BuildException if any error happens
     */
    public void initData() {
        // This method is intentionally empty
    }

    /**
     * This method cleans up the data initialized by the {@link #initData()} method. In the case when
     * <tt>initserver</tt> attribute is set to <tt>false</tt> or data was not initialized, does nothing. Subclasses must
     * implement this method to perform the actual task of cleaning-up the data.
     *
     * @throws BuildException if any error happens
     */
    public void cleanUpData() {
        // This method is intentionally empty
    }

    /**
     * This method sets the value of the <tt>timeout</tt> attribute, which specifies the amount of time to wait for
     * server to be started (in milliseconds).
     *
     * @param timeout the value of the <tt>timeout</tt> attribute to be set, in milliseconds
     *
     * @throws IllegalArgumentException if timeout is negative
     */
    public void setTimeout(final long timeout) {
        if (timeout < 0) {
            throw new IllegalArgumentException(
                "The parameter named [timeout] was expected to be >=0 , but was [" + timeout + "].");
        }

        this.timeout = timeout;
    }

    /**
     * This method returns the value of the <tt>timeout</tt> attribute, which specifies the amount of time to wait for
     * server to be started (in milliseconds).
     *
     * @return the value of the <tt>timeout</tt> attribute, in milliseconds
     */
    public long getTimeout() {
        return timeout;
    }

    /**
     * This method sets the value of the <tt>testurl</tt> attribute, which specifies the URL used to check whether the
     * server is running. If this attribute is set and the server is running, the server will not be started by the
     * framework.
     *
     * @param testURL the value of the <tt>testurl</tt> attribute to  be set
     */
    public void setTestURL(final URL testURL) {
        this.testURL = testURL;
    }

    /**
     * This method returns the value of the <tt>testurl</tt> attribute, which specifies URL used to check if the server
     * is running. If this attribute is set and the server is running, the server will not be started by the framework.
     *
     * @return the value of the <tt>testurl</tt> attribute
     */
    public URL getTestURL() {
        return testURL;
    }

    /**
     * This method is a utility for the reflective invocation of a setter method in a Container instance. It tries to
     * find a method with the given name and argument type in the given instance and in case found, invokes that method
     * using the given value as argument.
     *
     * @param instance   the instance on which the setter method shall be called
     * @param setterName the name of the setter method
     * @param valueType  the type of the argument in the setter method declaration
     * @param value      the value to be passed when invoking the setter method
     *
     * @throws IllegalArgumentException in case the given instance, setterName or valueType are <tt>null</tt>, or the
     *                                  given setterName is an empty(trim'd) String
     * @throws BuildException           in case the appropriate setter method was not found or threw an Throwable during
     *                                  invocation
     */
    private static void callSetterByReflection(final Object instance, final String setterName, final Class valueType,
                                               final Object value) {
        if (instance == null) {
            throw new IllegalArgumentException("The parameter named [instance] was null.");
        }
        if (setterName == null) {
            throw new IllegalArgumentException("The parameter named [setterName] was null.");
        }
        if (setterName.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [setterName] was an empty String.");
        }
        if (valueType == null) {
            throw new IllegalArgumentException("The parameter named [valueType] was null.");
        }

        final Class instanceType = instance.getClass();

        final Method method;
        try {
            method = instanceType.getMethod(setterName, new Class[]{valueType});
        } catch (NoSuchMethodException e) {
            throw new BuildException("The container [" + instance + "] of type [" + instanceType.getName()
                + "] did not have an appropriate setter method named [" + setterName + "] with argument of type ["
                + valueType.getName() + "].", e);
        }
        try {
            method.invoke(instance, new Object[]{value});
        } catch (Throwable e) {
            // catch of Throwable is normally discouraged by the TC coding style, but
            // as a lot of Errors can arise from a reflection invocation and these
            // errors don't need to be let pass through this catch clause (as they
            // definitely belong to the reflective invocation and not the normal program
            // execution) all Throwables are caught here and
            // wrapped into a BuildException.
            throw new BuildException("The container [" + instance + "] of type [" + instanceType.getName()
                + "] did throw an exception upon invocation of the setter method named [" + setterName
                + "] with argument of type [" + valueType.getName() + "], using the argument [" + value
                + "] in the invocation.", e);
        }
    }
}
