/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import com.topcoder.testframework.web.DefaultWebApplicationServer;
import junit.framework.TestCase;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;
import org.apache.tools.ant.taskdefs.optional.junit.BatchTest;
import org.apache.tools.ant.types.FileSet;

import java.io.File;
import java.util.List;


/**
 * This class contains the unit tests for {@link com.topcoder.testframework.ServerAppTestTask}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ServerAppTestTaskTest extends TestCase {
    /**
     * This is the instance to be tested. It is created in {@link #setUp()}.
     */
    private ServerAppTestTask testTask;

    /**
     * This method tests {@link com.topcoder.testframework.ServerAppTestTask#ServerAppTestTask()} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtor() throws Exception {
        final ServerAppTestTask instance = new ServerAppTestTask();
        assertEquals("constructor does not assign all fields", null, instance.getReportsDir());
    }

    /**
     * This method tests {@link ServerAppTestTask#setReportsDir(java.io.File)}  and {@link
     * ServerAppTestTask#getReportsDir()}for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetSetReportsDir() throws Exception {
        assertNull(testTask.getReportsDir());
        final File file = new File(".");
        testTask .setReportsDir(file);
        assertEquals(file, testTask.getReportsDir());
    }

    /**
     * This method tests {@link ServerAppTestTask#createDynamicElement(String, String, String)}  for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElement() throws Exception {
        final List testTaskServers = (List) TestingUtils.getFieldValue(testTask, "servers");

        assertEquals(0, testTaskServers.size());

        final Object element1 = testTask.createDynamicElement(null, "blah", null);
        assertNotNull(element1);
        assertTrue(ServerElement.class.isAssignableFrom(element1.getClass()));
        assertEquals(1, testTaskServers.size());
        assertTrue(testTaskServers.contains(element1));
        final ServerElement serverElement = (ServerElement) element1;
        assertNotNull(serverElement.getAntTaskInfo());
        assertNull(serverElement.getServer());
        assertEquals("blah", TestingUtils.getFieldValue(serverElement, "clazz"));

        //check multiple servers are possible
        final Object element2 = testTask.createDynamicElement(null, "blup", null);
        assertNotNull(element2);
        assertTrue(ServerElement.class.isAssignableFrom(element2.getClass()));
        assertEquals(2, testTaskServers.size());
        assertTrue(testTaskServers.contains(element1));
        assertTrue(testTaskServers.contains(element2));

    }

    /**
     * This method tests the correct failure behavior of {@link ServerAppTestTask#createDynamicElement(String, String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> localName given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureNull() throws Exception {
        try {
            testTask.createDynamicElement(null, null, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerAppTestTask#createDynamicElement(String, String,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: empty localName given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureEmpty() throws Exception {
        try {
            testTask.createDynamicElement(null, "  ", null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link ServerAppTestTask#execute()}  for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecute() throws Exception {
        final ServerElement serverElement =
            (ServerElement) testTask.createDynamicElement(null, DefaultWebApplicationServer.class.getName(), null);
        final DefaultWebApplicationServer webApplicationServer =
            (DefaultWebApplicationServer) serverElement.createDynamicElement("tomcat5x");

        webApplicationServer.setWarFile(new File(TestingUtils.VALID_PREPARED_WAR_FILE));
        webApplicationServer.setDir(new File(TestingUtils.getValidTomcatHomeDir()));
        webApplicationServer.setPort(TestingUtils.getPort());

        final BatchTest batchTest = testTask.createBatchTest();
        final FileSet set = new FileSet();
        set.setDir(new File("test_files/demo/src/tests"));
        set.setIncludes("**/*Test*.java");
        batchTest.addFileSet(set);

        testTask.setReportsDir(new File("test_files/demo/build"));
        testTask.execute();
    }

    /**
     * This method tests the correct failure behavior of {@link ServerAppTestTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: no server child element s have been specified
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteNoServersSpecified() throws Exception {
        try {
            testTask.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerAppTestTask#execute()}.
     * <p/>
     * <b>Failure reason</b>: server configuration is insufficient for server start
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testExecuteServerNotConfigured() throws Exception {
        final ServerElement serverElement =
            (ServerElement) testTask.createDynamicElement(null, DefaultWebApplicationServer.class.getName(), null);
        serverElement.createDynamicElement("tomcat5x");
        try {
            testTask.execute();
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        testTask = new ServerAppTestTask();
        final Project project = new Project();
        project.init();
        testTask.setProject(project);
        testTask.setLocation(Location.UNKNOWN_LOCATION);
        testTask.setTaskName("taskname");
        testTask.setOwningTarget(new Target());
    }
}
