/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import com.topcoder.testframework.web.DefaultWebApplicationServer;
import junit.framework.TestCase;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;


/**
 * This class contains the unit tests for {@link ServerElement}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ServerElementTest extends TestCase {
    /**
     * This is a server class name used in test cases.
     */
    private static final String NAME = DefaultWebApplicationServer.class.getName();
    /**
     * This is a valid argument for use with {@link ServerElement#createDynamicElement(String)}.
     */
    private static final String VALID_SERVER_ELEMENT_NAME = "tomcat5x";

    /**
     * This is an AntTaskInfo instance used in test cases, created in {@link #setUp()}.
     */
    private AntTaskInfo info;
    /**
     * This is the instance to be tested. It is created in {@link #setUp()}.
     */
    private ServerElement serverElement;

    /**
     * This method tests {@link ServerElement#ServerElement(String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtor() throws Exception {
        final ServerElement instance = new ServerElement(NAME);
        assertEquals("constructor does not assign all fields", NAME, TestingUtils.getFieldValue(instance, "clazz"));
        assertNull(instance.getServer());
        assertNull(instance.getAntTaskInfo());
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#ServerElement(String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> classname given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtorFailNullClassname() throws Exception {
        try {
            new ServerElement(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#ServerElement(String)}.
     * <p/>
     * <b>Failure reason</b>: empty classname given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCtorFailEmptyClassname() throws Exception {
        try {
            new ServerElement("  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link ServerElement#setAntTaskInfo(AntTaskInfo)}  and {@link
     * ServerElement#getAntTaskInfo()}for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testGetSetAntTaskInfo() throws Exception {
        assertNull(serverElement.getAntTaskInfo());
        serverElement .setAntTaskInfo(info);
        assertEquals(info, serverElement.getAntTaskInfo());
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#setAntTaskInfo(AntTaskInfo)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> AntTaskInfo given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testSetAntTaskInfoFailureNull() throws Exception {
        try {
            serverElement.setAntTaskInfo(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link ServerElement#createDynamicElement(String)}  and {@link ServerElement#getServer()}for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElement() throws Exception {
        serverElement.setAntTaskInfo(info);
        final Object element = serverElement.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
        assertNotNull(element);
        assertTrue(Server.class.isAssignableFrom(element.getClass()));

        assertEquals(element, serverElement.getServer());
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> name given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureNull() throws Exception {
        try {
            serverElement.createDynamicElement(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: empty name given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureEmpty() throws Exception {
        try {
            serverElement.createDynamicElement("  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: duplicate call to the method
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureDuplicateCall() throws Exception {
        serverElement.setAntTaskInfo(info);
        serverElement.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
        try {
            serverElement.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: the instance has no AntTaskInfo instance yet
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureNoAntTaskInfo() throws Exception {
        try {
            serverElement.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: class named in constructor does not exist
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureClassNotFound() throws Exception {
        final ServerElement serverElement1 = new ServerElement("blah");
        serverElement1.setAntTaskInfo(info);
        try {
            serverElement1.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: class named in constructor is not of type {@link Server}
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureClassWrongType() throws Exception {
        final ServerElement serverElement1 = new ServerElement(getClass().getName());
        serverElement1.setAntTaskInfo(info);
        try {
            serverElement1.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: class named in constructor has no appropriate constructor signature
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureClassNoMatchingConstructor() throws Exception {
        final ServerElement serverElement1 = new ServerElement(DummyServerWrongConstructor.class.getName());
        serverElement1.setAntTaskInfo(info);
        try {
            serverElement1.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link ServerElement#createDynamicElement(String)}.
     * <p/>
     * <b>Failure reason</b>: class named in constructor throws exception during constructor invocation
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCreateDynamicElementFailureClassThrowsExceptionInConstructor() throws Exception {
        final ServerElement serverElement1 = new ServerElement(DummyServerExceptionInConstructor.class.getName());
        serverElement1.setAntTaskInfo(info);
        try {
            serverElement1.createDynamicElement(VALID_SERVER_ELEMENT_NAME);
            fail("should throw");
        } catch (BuildException expected) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        info = new AntTaskInfo(new Project(), "task name", Location.UNKNOWN_LOCATION, new Target());
        serverElement = new ServerElement(NAME);
    }
}
