/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package demo;

import com.topcoder.testframework.DataServer;


/**
 * This class represents a custom server implementation that is defined inside the test code of the test project instead
 * of the testing framework main code. This server is called from the build file.
 * <p/>
 * This is an example on how custom servers can be added to the testing framework dynamically.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class CustomServer extends DataServer {
    /**
     * Creates a new CustomServer with the given type. This method is called from the testing framework.
     *
     * @param type the type parameter is the name of the child XML element inside the class-named element
     */
    public CustomServer(final String type) {
        super(type);
        System.out.println("CustomServer.CustomServer(" + type + ")");
    }

    /**
     * This method demonstrates how custom configuration attributes can be added to a custom server class.
     * <p/>
     * This method will be called by ant when the XML element in the build file has an attribute named
     * 'customAttribute'.
     *
     * @param attribute the value of the custom attribute
     */
    public void setCustomAttribute(final String attribute) {
        System.out.println("Ant gave CustomServer instance the custom attribute value: " + attribute);
    }

    /**
     * This method starts the server. In the case when <tt>startserver</tt> attribute is set to <tt>false</tt>, the
     * server should not be started, but some initialization may still be needed. Subclasses must implement this method
     * to perform the actual task of starting up the server.
     *
     * @throws org.apache.tools.ant.BuildException
     *          if any error happens
     */
    public void startUp() {
        System.out.println("CustomServer.startUp");
    }

    /**
     * This method stops the server. The server should not be stopped if it wasn't started by the startUp() method, i.e.
     * <tt>startserver</tt> attribute is set to <tt>false</tt>. Subclasses must implement this method to perform the
     * actual task of stopping the server.
     *
     * @throws org.apache.tools.ant.BuildException
     *          if any error happens
     */
    public void shutDown() {
        System.out.println("CustomServer.shutDown");
    }

    /**
     * This method initializes the server  with data. In the case when <tt>initserver</tt> attribute is set to
     * <tt>false</tt>, does nothing, or can check if the data are properly initialized, the implementation can choose
     * the preferred way. Subclasses must implement this method to perform the actual task of initializing server with
     * data.
     *
     * @throws org.apache.tools.ant.BuildException
     *          if any error happens
     */
    public void initData() {
        System.out.println("CustomServer.initData");
    }

    /**
     * This method cleans up the data initialized by the {@link #initData()} method. In the case when
     * <tt>initserver</tt> attribute is set to <tt>false</tt> or data was not initialized, does nothing. Subclasses must
     * implement this method to perform the actual task of  cleaning-up the data.
     *
     * @throws org.apache.tools.ant.BuildException
     *          if any error happens
     */
    public void cleanUpData() {
        System.out.println("CustomServer.cleanUpData");
    }
}
